from typing import Literal

from configs_core.base_classes import BaseSettingClass

from .configs.configs.services.base_python_service_settings import BasePythonServiceSettingsClass
from .configs.configs.settings.all_settings import SETTINGS
from .configs.configs.settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)


class AnalyticsSetting(BaseSettingClass):
    """
    Analytics settings

    Attributes:
        settings(dict): raw analytics settings
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        super().__init__(serviceName, requiredServiceName)
        self.settings = settings


class AgentUnitVideoSettings(BaseSettingClass):
    """
    Luna agent video processing settings.

    Attributes:
        deviceClass (Literal["cpu", "gpu", "auto"]): decoder device class
        workerCount (int): number of decoder workers
        maxSize (int): video max size, in megabytes
        timeouts (ServiceTimeoutsSettings): timeouts settings for video saver
        framePoolSize (int): number of pre-allocated frames for video stream
        ffmpegThreadCount (int): number of threads ffmpeg uses when decoding
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        super().__init__(serviceName, settingGroupName)
        self.deviceClass: Literal["cpu", "gpu", "auto"] = settings["decoder_device_class"]
        self.workerCount: int = settings["decoder_worker_count"]
        self.maxSize: int = settings["max_size"]
        self.timeouts: _ServiceTimeoutsSettings = _ServiceTimeoutsSettings(
            serviceName, settingGroupName, settings["timeouts"]
        )
        self.framePoolSize: int = settings["frame_pool_size"]
        self.ffmpegThreadCount: int = settings["ffmpeg_thread_count"]


class AgentSettings(BaseSettingClass):
    """
    Agent extended settings

    Attributes:
        maxActiveStreams (int): number of active streams
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.maxActiveStreams = settings["max_active_streams"]


class RuntimeSetting(BaseSettingClass):
    """
    Runtime settings

    Attributes:
        globalDeviceClass (str): execution device type - cpu, gpu
        numThreads (int): number of worker threads
        numComputeStreams (int): number of streams; Increases performance, but works only with new versions of
                                 NVIDIA driversnumber of streams; Increases performance, but works only with new
                                 versions of NVIDIA drivers
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.globalDeviceClass = settings["global_device_class"]
        self.numThreads = settings["num_threads"]
        self.numComputeStreams = settings["num_compute_streams"]


class ServiceSettings(BasePythonServiceSettingsClass):
    """
    Settings class for luna-lambda-agent-unit
    Attributes:
        videoSettings (AgentUnitVideoSettings): agent unit video settings
        analyticsSettings (RuntimeSetting): analytics settings
        runtimeSettings (RuntimeSetting): agent unit runtime settings
        videoManagerTimeouts (_ServiceTimeoutsSettings): luna video manager timeouts
        videoManagerAddress (_ServiceAddressSettings): luna video manager address
        lunaSenderTimeouts (_ServiceTimeoutsSettings): luna sender timeouts
        lunaSenderAddress (_ServiceAddressSettings): luna sender address
        licensesAddress (_ServiceAddressSettings): luna licenses address
        objectsStorageTimeouts (_ServiceTimeoutsSettings): luna objects storage timeouts
        objectsStorage (_ImageStoreAddressSettings): luna objects storages address
        imageOriginStorageTimeouts (_ServiceTimeoutsSettings): luna image origin storage timeouts
        imageOriginStorage (_ImageStoreAddressSettings): luna image origin storage address
        lunaEventsTimeouts (_ServiceTimeoutsSettings): luna events timeouts
        lunaEventsAddress (_ServiceAddressSettings): luna events address
        additionalServicesUsage (_AdditionalServicesUsageSetting): additional services usages
        externalApiAddress (_ServiceAddressSettings): external luna api address
        agentSettings (AgentSettings): agent settings
    """

    serviceName = "luna-lambda-agent-unit"
    settingsStore = SETTINGS

    def __init__(self):
        super().__init__()
        self.videoSettings: AgentUnitVideoSettings | None = None
        self.analyticsSettings: RuntimeSetting | None = None
        self.runtimeSettings: RuntimeSetting | None = None
        self.videoManagerTimeouts: _ServiceTimeoutsSettings | None = None
        self.videoManagerAddress: _ServiceAddressSettings | None = None
        self.lunaSenderTimeouts: _ServiceTimeoutsSettings | None = None
        self.lunaSenderAddress: _ServiceAddressSettings | None = None
        self.licensesAddress: _ServiceAddressSettings | None = None
        self.objectsStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.objectsStorage: _ImageStoreAddressSettings | None = None
        self.imageOriginStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.imageOriginStorage: _ImageStoreAddressSettings | None = None
        self.lunaEventsTimeouts: _ServiceTimeoutsSettings | None = None
        self.lunaEventsAddress: _ServiceAddressSettings | None = None
        self.additionalServicesUsage: _AdditionalServicesUsageSetting | None = None
        self.externalApiAddress: _ServiceAddressSettings | None = None
        self.agentSettings: AgentSettings | None = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()

        self.videoSettings = AgentUnitVideoSettings(
            self.serviceName, "LUNA_LAMBDA_AGENT_UNIT_VIDEO_SETTINGS", self["LUNA_LAMBDA_AGENT_UNIT_VIDEO_SETTINGS"]
        )
        self.runtimeSettings = RuntimeSetting(
            self.serviceName,
            "LUNA_LAMBDA_AGENT_UNIT_RUNTIME_SETTINGS",
            self["LUNA_LAMBDA_AGENT_UNIT_RUNTIME_SETTINGS"],
        )
        self.analyticsSettings = AnalyticsSetting(
            self.serviceName,
            "LUNA_LAMBDA_AGENT_UNIT_ANALYTICS_SETTINGS",
            self["LUNA_LAMBDA_AGENT_UNIT_ANALYTICS_SETTINGS"],
        )
        self.videoManagerTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_VIDEO_MANAGER_TIMEOUTS", self["LUNA_VIDEO_MANAGER_TIMEOUTS"]
        )
        self.videoManagerAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_VIDEO_MANAGER_ADDRESS", self["LUNA_VIDEO_MANAGER_ADDRESS"]
        )
        self.lunaSenderTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_SENDER_TIMEOUTS", self["LUNA_SENDER_TIMEOUTS"]
        )
        self.lunaSenderAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_SENDER_ADDRESS", self["LUNA_SENDER_ADDRESS"]
        )
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.objectsStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_IMAGE_STORE_OBJECTS_TIMEOUTS", self["LUNA_IMAGE_STORE_OBJECTS_TIMEOUTS"]
        )
        self.objectsStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_OBJECTS_ADDRESS", self["LUNA_IMAGE_STORE_OBJECTS_ADDRESS"]
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS", self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"]
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_IMAGES_ADDRESS", self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"]
        )
        self.lunaEventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_EVENTS_TIMEOUTS", self["LUNA_EVENTS_TIMEOUTS"]
        )
        self.lunaEventsAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_EVENTS_ADDRESS", self["LUNA_EVENTS_ADDRESS"]
        )
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )
        self.agentSettings = AgentSettings(
            self.serviceName,
            "LUNA_LAMBDA_AGENT_UNIT_SETTINGS",
            self["LUNA_LAMBDA_AGENT_UNIT_SETTINGS"],
        )

        return self
