import importlib
import sys
from functools import partial
from socket import socket
from typing import Literal, Type

from cow.manager.process import SanicProcess
from cow.manager.utils import createManager
from cow.web.configs_handler import ConfigsHandler

from luna_lambda_tools import LambdaUserHandler

from .agent_unit_config import ServiceSettings as LambdaAgentServiceSettings
from .agent_utils.utils import updateAgentModules
from .base_application import BaseAppCtx
from .base_application_agent import AgentLambdaApp
from .base_application_handlers import HandlersLambdaApp
from .base_application_standalone import StandaloneLambdaApp
from .base_application_tasks import LambdaTasksApp
from .config import DEFAULT_PORT
from .handlers import Handler, HealthCheckHandler, SpecDocHandler
from .helpers import getParsedCMDArgs
from .task_unit_config import ServiceSettings as LambdaTaskServiceSettings
from .unit_config import ServiceSettings as LambdaServiceSettings

try:
    updateAgentModules()
except ImportError:
    pass


def getAppCls(lambdaType: Literal["standalone", "handlers", "tasks", "agent"]):
    """Get app class by lambda type"""
    if lambdaType == "tasks":
        appCls = LambdaTasksApp
    elif lambdaType == "handlers":
        appCls = HandlersLambdaApp
    elif lambdaType == "agent":
        appCls = AgentLambdaApp
    else:
        appCls = StandaloneLambdaApp
    return appCls


def extractAdditionalRoutes() -> list[tuple[str, Type]]:
    """Extract additional routes from lambda"""
    additionalRoutes = getattr(importlib.import_module("lambda_main", "main"), "ROUTES_TO_ADD", [])
    routes = []
    for handler in additionalRoutes:
        if issubclass(handler, LambdaUserHandler) and handler.__module__.endswith("lambda_main"):
            routes.append((handler.route, handler))
    return routes


def addRoutes(application: StandaloneLambdaApp | HandlersLambdaApp | LambdaTasksApp):
    """Add route to app by lambda type"""
    routes = [
        (f"/healthcheck", HealthCheckHandler),
        (f"/docs/spec", SpecDocHandler),
        (f"/config", ConfigsHandler),
    ]
    if isinstance(application, (StandaloneLambdaApp, HandlersLambdaApp)):
        routes.append(("/main", Handler))
    routes.extend(extractAdditionalRoutes())
    application.addRoutes(routes=routes)
    return application


def getSettingsClass(lambdaType: Literal["standalone", "handlers", "tasks", "agent"]):
    """Get settings class by lambda type"""
    if lambdaType == "tasks":
        return LambdaTaskServiceSettings
    if lambdaType == "agent":
        return LambdaAgentServiceSettings
    return LambdaServiceSettings


def _runLambda(sock: socket, lambdaType: Literal["standalone", "handlers", "tasks", "agent"]):
    """Run lambda"""
    lambdaName, ctxClass = getLambdaNameNCtx(lambdaType=lambdaType)

    application = getAppCls(lambdaType=lambdaType)
    application.ctxCls = ctxClass
    application = application()
    addRoutes(application=application)
    application.run(sock=sock)


def getCtxClass(
    lambdaType: Literal["standalone", "handlers", "tasks", "agent"], lambdaVersion: int, lambdaName: str
) -> Type[BaseAppCtx]:
    """Get ctx class by lambda type"""
    if lambdaType == "standalone":
        from .base_application_standalone import AppCtx
    elif lambdaType == "handlers":
        from .base_application_handlers import AppCtx
    elif lambdaType == "agent":
        from .base_application_agent import AppCtx
    else:
        from .base_application_tasks import AppCtx

    AppCtx.lambdaVersion = lambdaVersion
    AppCtx.lambdaName = lambdaName
    return AppCtx


def getLambdaNameNVersion(lambdaType: Literal["standalone", "handlers", "tasks", "agent"]) -> tuple[str, int]:
    """Get lambda name and version"""
    cmdArgs = getParsedCMDArgs(lambdaType=lambdaType)
    lambdaName: str = cmdArgs.name
    if cmdArgs.name.startswith("lambda-"):
        lambdaName = cmdArgs.name[len("lambda-") :]
    return lambdaName, cmdArgs.version


def getLambdaNameNCtx(lambdaType: Literal["standalone", "handlers", "tasks", "agent"]) -> tuple[str, Type[BaseAppCtx]]:
    """Get lambda context"""
    lambdaName, lambdaVersion = getLambdaNameNVersion(lambdaType=lambdaType)
    return lambdaName, getCtxClass(lambdaType=lambdaType, lambdaVersion=lambdaVersion, lambdaName=lambdaName)


def run(lambdaType: Literal["standalone", "handlers", "tasks", "agent"]):
    """Run lambda using process manager"""
    if lambdaType == "agent":
        try:
            from luna_smith.run import manageApp
        except ImportError:
            raise RuntimeError("Lambda-agent required `luna-smith` package")
        lambdaName, lambdaVersion = getLambdaNameNVersion(lambdaType=lambdaType)
        setattr(
            sys.modules["app.version"],
            "VERSION",
            {"Version": {"api": lambdaVersion, "major": lambdaVersion, "minor": 0, "patch": 0}},
        )
        manageApp(name=lambdaName)
    else:
        lambdaName, ctxClass = getLambdaNameNCtx(lambdaType=lambdaType)
        target = partial(_runLambda, lambdaType=lambdaType)
        processes = [SanicProcess(name=lambdaName, target=target)]

        manager = createManager(
            processes,
            configClass=getSettingsClass(lambdaType=lambdaType),
            ctxClass=ctxClass,
            port=DEFAULT_PORT,
        )

        manager.run()
